#include	<p18f26k22.h>
#include "Delay16M/Delays_16Mhz.h"
/************************************************************************ 
*   LCD graphics driver for Digole 12864w with ST7920 driver using       * 
*   CCS software. May work with other ST7920 driven LCD. It has          * 
*   the following Pin assignments                                        * 
*                                                                        * 
*   Pin 1 ------------> Gnd                                              *    
*   Pin 2 ------------> +5volts                                          * 
*   Pin 3 ------------> Contrast                                         * 
*   Pin 4 ------------> Register Select                                  * 
*   Pin 5 ------------> Read/Write                                       * 
*   Pin 6 ------------> Enable                                           * 
*   Pin 7-14 ---------> Data bits                                        * 
*   Pin 15 -----------> PSB (parallel=high & serial=low)                 * 
*   Pin 16 -----------> NoConnection                                     * 
*   Pin 17 -----------> Reset                                            * 
*   Pin 18 -----------> Vout                                             * 
*   Pin 19 -----------> +5volts                                          * 
*   Pin 20 -----------> Gnd                                              * 
*                                                                        * 
*   This driver is using 8bit parallel operation for optimal RAM         * 
*   requirements, if you want i can change it to 4 bit operation.        * 
*                                                                        * 
*   Developed by Christian Rebogio                                       * 
*   suggestions and request please email me at                           * 
*   christian.rebogio@gmail.com                                          * 
*                                                                        * 
* READ THE DATASHEET FOR ST7920!!!!!!!                                   * 
************************************************************************/ 
// change the pin assignment depending on your circuit 

//#define  rs  PIN_C0                    //COMMNAD/DATA SELECT 
//#define  rw  PIN_C1                    //READ/WRITE SELECT              
//#define  e   PIN_C2                    //ENABLE SIGNAL                  
//#define  rst PIN_C5                    //RESET SIGNAL    

#define   e    LATCbits.LATC0        
#define   rw    LATCbits.LATC1
#define   rs      LATCbits.LATC2  // DI  Register Select pin4
#define   rst   LATCbits.LATC3  // /ReSeT pin17
#define   output_b  LATB
#define   LCD_Data  LATB
#define input(PIN_B7)  PORTBbits.RB7

extern unsigned char image[];

#define ON   1 
#define OFF   0 
#define XVAL 16  // 16 X 16 or 256 for there is 8 word values for the upper and lower 
#define YVAL 32 

// PSB is tied to Vcc for this driver because this driver uses Parallel  operation. 
// data is sent using port B so change output_b() to other ports you 
//want to use. Dont for get to change the Busy pin @ lcd_check_busy 

#define GLCD_WIDTH   128 



////////////////////////////////////////////////////////////////////////////////// 
//The following are the functions included in this driver file 
// glcd_readbyte(); 
// glcd_instruction( instruction ); 
// glcd_data( data ); - data can be an array of characters! 
// glcd_check_busy(); 
// glcd_update(); -must be called always after writing a pixel or using functions 
//                 from GRAPHICS.C .. Only applicaticable in Graphing mode 
// glcd_fillscreen( ON or OFF); 
// glcd_init_graph(); initialize for graphing mode 
// glcd_init_basic(); initilize for accessing the stored Characters 
//                     you can use glcd_data() for writing text 
// glcd_pixel(x coordinate, y coordinate, ON or OFF);  
//            -WORKS WITH GRAPHIC.C  from CCS Drivers 
// glcd_plot_image(width,height,X coor, Y coor, inverse); 
//            -plots the image[] array. Declare it first before this driver. 
//             or modify this driver 
// 
////////////////////////////////////////////////////////////////////////////////// 



typedef union 
{ 
  unsigned int word; 
  unsigned char nbyte[2]; 
} Dots; 

typedef struct 
{ 
  int1 refresh; 
  Dots pix[YVAL][XVAL];   // Max dimensions for display (x,y) = (128,32) 
  } GD_RAM;             //  (0,0) corresponds to upper lefthand corner. 

GD_RAM gdram; 


unsigned unsigned char glcd_readByte (unsigned int address) 
{ 
  unsigned unsigned char data;   // Stores the data read from the LCD 
  if(address==1){ 
     output_high(rs); 
  } 
  if(address==0){ 
     output_low(rs); 
  } 
  output_high(rw);//GLCD_RW = RW_READ;      // Set for reading 
  output_high(e);//GLCD_E = 1;      // Pulse the enable pin 
  Delay_uS(1); 
  data=input_b();      // Get the data from the display's output register 
  output_low(e);//GLCD_E = 0; 
  return (data); 
} 

  
void glcd_check_busy(){ 
   int1 busy=1; 
   rs=0;      // LOW RS and High RW will put the lcd to 
   rw=1;      // read busy flag and address counter 
   while(busy){         // will cycle until busy flag is 0 
      e=1; 
      if(!input(PIN_B7)){ 
         busy=0; 
      } 
      e=0; 
   } 
} 
  
void glcd_instruction(unsigned char x){ 
   glcd_check_busy();      //must be satisfied before sending instruction 
   rs=0;      // LOW RS and LOW RW will put the lcd to 
   rw=0;      // Write instruction mode 
   LCD_Data=x;         // 8bit data to bus 
   e=1;      // enable 
   Delay_uS(1);        
   e=0;      // disable 
} 
void glcd_data(unsigned char x){ 
   glcd_check_busy(); 
   rs=1;      // HIGH RS and LOW RW will put the lcd to 
   rw=0;      // Write data register mode 
   LCD_Data=x; 
   e=1; 
   Delay_uS(1); 
   e=0; 
} 
  

void glcd_fillScreen (unsigned int color) 
{ 
  unsigned char v, h; 
  unsigned int d; 
  d = (color == ON ? 0xFFFFL : 0x0000L); 
  for (v=0; v < YVAL; v++) 
  { 
    for (h=0; h < XVAL; h++) 
    { 
      gdram.pix[v][h].word = d; 
    } 
  } 
  gdram.refresh = 1; 
} 


void glcd_update () 
{ 
  unsigned char v, h; 


  if (gdram.refresh) 
  { 
    for (v=0; v <YVAL; v++) 
    { 
      glcd_instruction( 0x80 | v);   // Set Vertical Address. 
      glcd_instruction( 0x80 | 0);   // Set Horizontal Address. 

      for (h=0; h <XVAL; h++) 
      { 
        glcd_data( gdram.pix[v][h].nbyte[1]);   // Write High Byte. 
        glcd_data(  gdram.pix[v][h].nbyte[0]);   // Write Low Byte. 
      } 
    } 
    gdram.refresh = 0; 
  } 
} 

void glcd_init_graph(){ 
   Delay_mS(40); 
   rst=0;         //reset LCD 
   Delay_uS(10);                      
   rst=1;        //LCD normal operation 
   glcd_instruction(0x30);   //set 8 bit operation and basic instruction set 
   Delay_uS(144); 
   glcd_instruction(0x0C);   //display on cursor off and char blink off 
   Delay_uS(100); 
   glcd_instruction(0x01);   //display clear 
   Delay_mS(10); 
   glcd_instruction(0x06);   //entry mode set 
   Delay_uS(72);                  
  glcd_instruction(0x34);    // Select extended instruction set. 
  Delay_mS (10); 
  glcd_instruction(0x36);    // Graphic display ON. 
  Delay_mS (10); 

  glcd_fillScreen (OFF); 
  glcd_update (); 

} 

void glcd_init_basic(){ 
   Delay_mS(40); 
   rst=0;         //reset LCD 
   Delay_uS(1);                      
   rst=1;        //LCD normal operation 
   glcd_instruction(0x30);   //set 8 bit operation and basic instruction set 
   Delay_uS(160); 
   glcd_instruction(0x0C);   //display on cursor off and char blink off 
   Delay_uS(100); 
   glcd_instruction(0x01);   //display clear 
   Delay_mS(10); 
   glcd_instruction(0x06);   //entry mode set 
   Delay_uS(80);                  
} 

void glcd_pixel(unsigned char x, unsigned char y, int color) 
{ 
  unsigned char v, h, b; 
  if(y>31){x += 128; y-= 32;}; 
  v = y; 
  h = x/16; 
  b = 15 - (x%16); 
  
  if (color == ON) bit_set (gdram.pix[v][h].word, b); 
  else bit_clear (gdram.pix[v][h].word, b); 

  gdram.refresh = 1; 
} 

void glcd_plot_image(int width,int height,int x,int y,int inverse) 
{ 
   unsigned int i=0, j=0, k=0; 
   unsigned unsigned int count=0; 
   //glcd_fillScreen(OFF);                        //Clears the screen (opt.) 
   for(j=0;j<height;j++) 
      {    
         for(;i<width;) 
         { 
            for(k=8;k>0;k--){ 
               if(inverse)glcd_pixel(i+x,j+y,~bit_test(image[count],(k-1))); 
               else glcd_pixel(i+x,j+y,bit_test(image[count],(k-1))); 
                
               i++; 
            } 
            count++; 
         } 
      i=0; 
      } 
} 


